@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/SOLARIS/osi_vnodeops.c,v 1.6 2001/07/12 19:58:21 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/SOLARIS/osi_vnodeops.c,v 1.7.2.2 2002/04/24 01:41:36 zacheiss Exp $");
 
 #if	defined(AFS_SUN_ENV) || defined(AFS_SUN5_ENV)
 /*
@@ -125,10 +125,10 @@ int afs_vmread(avp, auio, ioflag, acred)
 {
     register int code;
 
-    if (!RW_READ_HELD(&((struct vcache *)avp)->rwlock))
+    if (!RW_READ_HELD(&(VTOAFS(avp))->rwlock))
 	osi_Panic("afs_vmread: !rwlock");
     AFS_GLOCK();
-    code = afs_nfsrdwr((struct vcache *)avp, auio, UIO_READ, ioflag, acred);
+    code = afs_nfsrdwr(VTOAFS(avp), auio, UIO_READ, ioflag, acred);
     AFS_GUNLOCK();
     return code;
 }
@@ -142,10 +142,10 @@ int afs_vmwrite(avp, auio, ioflag, acred)
 {
     register int code;
 
-    if (!RW_WRITE_HELD(&((struct vcache *)avp)->rwlock))
+    if (!RW_WRITE_HELD(&(VTOAFS(avp))->rwlock))
 	osi_Panic("afs_vmwrite: !rwlock");
     AFS_GLOCK();
-    code = afs_nfsrdwr((struct vcache *)avp, auio, UIO_WRITE, ioflag, acred);
+    code = afs_nfsrdwr(VTOAFS(avp), auio, UIO_WRITE, ioflag, acred);
     AFS_GUNLOCK();
     return code;
 }
@@ -184,7 +184,7 @@ struct AFS_UCRED *acred;
 
 #if	defined(AFS_SUN56_ENV)
     if (len <= PAGESIZE)
-	code = afs_GetOnePage((struct vnode *) vp, toff, len, protp, pl, plsz,
+	code = afs_GetOnePage(vp, toff, len, protp, pl, plsz,
 			      seg, addr, rw, acred);
 #else
 #ifdef	AFS_SUN5_ENV
@@ -198,7 +198,7 @@ struct AFS_UCRED *acred;
 #endif
 #endif
     else {
-	struct vcache *vcp = (struct vcache *)vp;
+	struct vcache *vcp = VTOAFS(vp);
 #ifdef	AFS_SUN5_ENV
 	ObtainWriteLock(&vcp->vlock, 548);
 	vcp->multiPage++;
@@ -206,10 +206,10 @@ struct AFS_UCRED *acred;
 #endif
 	afs_BozonLock(&vcp->pvnLock, vcp);
 #if	defined(AFS_SUN56_ENV)
-	code = pvn_getpages(afs_GetOnePage, (struct vnode *) vp, toff, 
+	code = pvn_getpages(afs_GetOnePage, vp, toff, 
 			    len, protp, pl, plsz, seg, addr, rw, acred);
 #else
-	code = pvn_getpages(afs_GetOnePage, (struct vnode *) vp, (u_int)off, 
+	code = pvn_getpages(afs_GetOnePage, vp, (u_int)off, 
 			    len, protp, pl, plsz, seg, addr, rw, acred);
 #endif
 	afs_BozonUnlock(&vcp->pvnLock, vcp);
@@ -265,10 +265,6 @@ struct AFS_UCRED *acred;
     afs_int32       toffset;
 #endif
 
-    if (!pl) return 0;			/* punt asynch requests */
-
-    len = PAGESIZE;
-    pl[0] = NULL;		 	/* Make sure it's empty */
     if (!acred) 
 #ifdef	AFS_SUN5_ENV
 	osi_Panic("GetOnePage: !acred");
@@ -276,8 +272,35 @@ struct AFS_UCRED *acred;
 	acred = u.u_cred;		/* better than nothing */
 #endif
 
+    avc = VTOAFS(vp);	/* cast to afs vnode */
+
+#ifdef	AFS_SUN5_ENV
+    if (avc->credp /*&& AFS_NFSXLATORREQ(acred)*/ && AFS_NFSXLATORREQ(avc->credp)) {
+	acred = avc->credp;
+    }
+#endif
+    if (code = afs_InitReq(&treq, acred)) return code;
+
+    if (!pl) {
+	/*
+	 * This is a read-ahead request, e.g. due to madvise.
+	 */
+	tdc = afs_GetDCache(avc, (afs_int32)off, &treq, &offset, &nlen, 1);
+	if (!tdc) return 0;
+
+	if (!(tdc->flags & DFNextStarted)) {
+	    ObtainReadLock(&avc->lock);
+	    afs_PrefetchChunk(avc, tdc, acred, &treq);
+	    ReleaseReadLock(&avc->lock);
+	}
+	afs_PutDCache(tdc);
+	return 0;
+    }
+
+    len = PAGESIZE;
+    pl[0] = NULL;		 	/* Make sure it's empty */
+
     /* first, obtain the proper lock for the VM system */
-    avc = (struct vcache *) vp;	/* cast to afs vnode */
 
     /* if this is a read request, map the page in read-only.  This will
      * allow us to swap out the dcache entry if there are only read-only
@@ -290,12 +313,6 @@ struct AFS_UCRED *acred;
 	mapForRead = 1;
 
     if (protp) *protp = PROT_ALL;
-#ifdef	AFS_SUN5_ENV
-    if (avc->credp /*&& AFS_NFSXLATORREQ(acred)*/ && AFS_NFSXLATORREQ(avc->credp)) {
-	acred = avc->credp;
-    }
-#endif
-    if (code = afs_InitReq(&treq, acred)) return code;
 #ifndef	AFS_SUN5_ENV
     if (AFS_NFSXLATORREQ(acred)) {
 	if (rw == S_READ) {
@@ -323,7 +340,7 @@ retry:
     }
 
     afs_BozonLock(&avc->pvnLock, avc);
-    ObtainSharedLock(&avc->lock,566);
+    ObtainReadLock(&avc->lock);
 
     afs_Trace4(afs_iclSetp, CM_TRACE_PAGEIN, ICL_TYPE_POINTER, (afs_int32) vp,
 	       ICL_TYPE_LONG, (afs_int32) off, ICL_TYPE_LONG, (afs_int32) len,
@@ -337,7 +354,7 @@ retry:
      * the locks and try again when the VM purge is done. */
     ObtainWriteLock(&avc->vlock, 550);
     if (avc->activeV) {
-	ReleaseSharedLock(&avc->lock); 
+	ReleaseReadLock(&avc->lock); 
 	ReleaseWriteLock(&avc->vlock); 
 	afs_BozonUnlock(&avc->pvnLock, avc);
     	afs_PutDCache(tdc);
@@ -359,7 +376,7 @@ retry:
     /* Check to see whether the cache entry is still valid */
     if (!(avc->states & CStatd)
 	|| !hsame(avc->m.DataVersion, tdc->f.versionNo)) {
-	ReleaseSharedLock(&avc->lock); 
+	ReleaseReadLock(&avc->lock); 
 	afs_BozonUnlock(&avc->pvnLock, avc);
 	afs_PutDCache(tdc);
 	goto retry;
@@ -438,19 +455,17 @@ retry:
 	    buf->b_blkno = btodb(toffset);
 	    bp_mapin(buf);		/* map it in to our address space */
 #ifndef	AFS_SUN5_ENV    
-	    ReleaseSharedLock(&avc->lock);
+	    ReleaseReadLock(&avc->lock);
 #endif
 #if	defined(AFS_SUN5_ENV)
 	    AFS_GLOCK();
-	    UpgradeSToWLock(&avc->lock, 564);
 	    code = afs_ustrategy(buf, acred);	/* do the I/O */
-            ConvertWToSLock(&avc->lock);
 	    AFS_GUNLOCK();
 #else
 	    code = afs_ustrategy(buf);	/* do the I/O */
 #endif
 #ifndef	AFS_SUN5_ENV    
-	    ObtainSharedLock(&avc->lock,245);
+	    ObtainReadLock(&avc->lock);
 #endif
 #ifdef	AFS_SUN5_ENV
 	    /* Before freeing unmap the buffer */
@@ -495,8 +510,13 @@ retry:
 
     AFS_GLOCK();
     pl[slot] = (struct page *) 0;
+    /*
+     * XXX This seems kind-of wrong: we shouldn't be modifying
+     *     avc->states while not holding the write lock (even
+     *     though nothing really uses CHasPages..)
+     */
     avc->states |= CHasPages;
-    ReleaseSharedLock(&avc->lock);
+    ReleaseReadLock(&avc->lock);
 #ifdef	AFS_SUN5_ENV
     ObtainWriteLock(&afs_xdcache,246);
     if (!mapForRead) {
@@ -526,7 +546,7 @@ retry:
     for(i=0; i<slot; i++)
 	PAGE_RELE(pl[i]);
 #endif
-    ReleaseSharedLock(&avc->lock);
+    ReleaseReadLock(&avc->lock);
     afs_BozonUnlock(&avc->pvnLock, avc);
 #ifdef	AFS_SUN5_ENV
     afs_PutDCache(tdc);
@@ -568,7 +588,7 @@ int afs_putpage(vp, off, len, flags, cred)
     afs_Trace4(afs_iclSetp, CM_TRACE_PAGEOUT, ICL_TYPE_POINTER, (afs_int32) vp,
 	       ICL_TYPE_LONG, (afs_int32) off, ICL_TYPE_LONG, (afs_int32) len,
 	       ICL_TYPE_LONG, (int) flags);
-    avc = (struct vcache *) vp;
+    avc = VTOAFS(vp);
     afs_BozonLock(&avc->pvnLock, avc);
     ObtainWriteLock(&avc->lock,247);
 
@@ -632,7 +652,7 @@ int afs_putapage(struct vnode *vp, struct page *pages,
                  int flags, struct AFS_UCRED *credp)
 {
     struct buf *tbuf;
-    struct vcache *avc = (struct vcache *)vp;
+    struct vcache *avc = VTOAFS(vp);
     afs_int32 code = 0;
     u_int toff, tlen = PAGESIZE, off = (pages->p_offset/PAGESIZE)*PAGESIZE;
     u_int poff = pages->p_offset;
@@ -700,7 +720,7 @@ struct AFS_UCRED *cred;
     AFS_STATCNT(afs_putpage);
     wholeEnchilada = (off == 0 && len == 0 && (flags & (B_INVAL|B_ASYNC)) == B_INVAL);
 
-    avc = (struct vcache *) vp;
+    avc = VTOAFS(vp);
     afs_BozonLock(&avc->pvnLock, avc);
     ObtainWriteLock(&avc->lock,248);
 
@@ -1045,9 +1065,9 @@ struct AFS_UCRED *acred;
 	ReleaseWriteLock(&avc->lock);	/* uiomove may page fault */
 	AFS_GUNLOCK();
 #if	defined(AFS_SUN56_ENV)
-	data = segmap_getmap(segkmap,(struct vnode *)avc,(u_offset_t)pageBase);
+	data = segmap_getmap(segkmap,AFSTOV(avc),(u_offset_t)pageBase);
 #else
-	data = segmap_getmap(segkmap, (struct vnode *) avc, pageBase);
+	data = segmap_getmap(segkmap, AFSTOV(avc), pageBase);
 #endif
 #ifndef	AFS_SUN5_ENV
 	code = as_fault(&kas, data+pageOffset, tsize, F_SOFTLOCK, mode);
@@ -1199,7 +1219,7 @@ struct AFS_UCRED *cred;
        struct segvn_crargs crargs;
 	register afs_int32 code;
 	struct vrequest treq;
-	register struct vcache *avc = (struct vcache *) vp;
+	register struct vcache *avc = VTOAFS(vp);
 
 	AFS_STATCNT(afs_map);
 
@@ -1260,7 +1280,7 @@ struct AFS_UCRED *cred;
 	} else
 	    (void) as_unmap(as, *addr, len);	/* unmap old address space use */
 	/* setup the create parameter block for the call */
-	crargs.vp = (struct vnode *) avc;
+	crargs.vp = AFSTOV(avc);
 	crargs.offset = (u_int)off;
 	crargs.cred = cred;
 	crargs.type = flags&MAP_TYPE;
@@ -1374,7 +1394,7 @@ void afs_rwlock(vnp, wlock)
     struct vnode *vnp;
     int wlock;
 {
-    rw_enter(&((struct vcache *)vnp)->rwlock, (wlock ? RW_WRITER : RW_READER));
+    rw_enter(&(VTOAFS(vnp))->rwlock, (wlock ? RW_WRITER : RW_READER));
 }
 
 
@@ -1382,7 +1402,7 @@ void afs_rwunlock(vnp, wlock)
     struct vnode *vnp;
     int wlock;
 {
-    rw_exit(&((struct vcache *)vnp)->rwlock);
+    rw_exit(&(VTOAFS(vnp))->rwlock);
 }
 
 
@@ -1399,7 +1419,11 @@ afs_seek(vnp, ooff, noffp)
     return code;
 }
 
-int afs_frlock(vnp, cmd, ap, flag, off, credp)
+int afs_frlock(vnp, cmd, ap, flag, off,
+#ifdef AFS_SUN59_ENV
+	       flkcb,
+#endif
+	       credp)
     struct vnode *vnp;
     int cmd;
 #if	defined(AFS_SUN56_ENV)
@@ -1409,6 +1433,9 @@ int afs_frlock(vnp, cmd, ap, flag, off, credp)
 #endif
     int flag;
     offset_t off;
+#ifdef AFS_SUN59_ENV
+    struct flk_callback *flkcb;
+#endif
     struct AFS_UCRED *credp;
 {
     register afs_int32 code = 0;
@@ -1416,6 +1443,10 @@ int afs_frlock(vnp, cmd, ap, flag, off, credp)
      * Implement based on afs_lockctl
      */
     AFS_GLOCK();
+#ifdef AFS_SUN59_ENV
+    if (flkcb)
+	afs_warn("Don't know how to deal with flk_callback's!\n");
+#endif
     if ((cmd == F_GETLK) || (cmd == F_O_GETLK) || (cmd == F_SETLK) || (cmd ==  F_SETLKW)) {
 #ifdef	AFS_SUN53_ENV
 	ap->l_pid = ttoproc(curthread)->p_pid;
@@ -1435,7 +1466,7 @@ int afs_frlock(vnp, cmd, ap, flag, off, credp)
 	AFS_GLOCK();
     }
 
-    code = afs_lockctl((struct vcache *)vnp, ap, cmd, credp);
+    code = afs_lockctl(VTOAFS(vnp), ap, cmd, credp);
     AFS_GUNLOCK();
     return code;
 }
@@ -1466,7 +1497,7 @@ int afs_space(vnp, cmd, ap, flag, off, credp)
 	if (!ap->l_len) {
 	    vattr.va_mask = AT_SIZE;
 	    vattr.va_size = ap->l_start;
-	    code = afs_setattr((struct vcache *)vnp, &vattr, 0, credp);
+	    code = afs_setattr(VTOAFS(vnp), &vattr, 0, credp);
 	} 
 	AFS_GUNLOCK();
     }
@@ -1513,9 +1544,16 @@ struct cred *credp;
     return EINVAL;
 }
 
-int  afs_dumpctl(vp, i)
+int  afs_dumpctl(vp, i
+#ifdef AFS_SUN59_ENV
+		 , blkp
+#endif
+		 )
 struct vnode *vp;
 int i;
+#ifdef AFS_SUN59_ENV
+int *blkp;
+#endif
 {
     afs_warn("afs_dumpctl: Not implemented\n");
     return EINVAL;
@@ -1858,7 +1896,7 @@ gafs_fsync(avc, acred)
 
 void afs_inactive(struct vcache *avc, struct AFS_UCRED *acred)
 {
-    struct vnode *vp = (struct vnode *)avc;
+    struct vnode *vp = AFSTOV(avc);
     if (afs_shuttingdown) return ;
 
     /*
