@@ -22,7 +22,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.4.4.7 2002/04/24 01:41:41 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.4.4.8 2002/06/08 00:13:41 shadow Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -356,7 +356,7 @@ tagain:
 	char *unlname = newname();
 
 	ReleaseWriteLock(&adp->lock);
-	code = afsrename(adp, aname, adp, unlname, acred);
+	code = afsrename(adp, aname, adp, unlname, acred, &treq);
 	Tnam1 = unlname;
 	if (!code) {
 	    tvc->mvid = (struct VenusFid *)unlname;
