@@ -18,7 +18,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_volume.c,v 1.6.2.2 2001/10/13 04:20:24 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_volume.c,v 1.6.2.4 2002/04/02 02:05:15 shadow Exp $");
 
 #include "../afs/stds.h"
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
@@ -225,6 +225,7 @@ void afs_ResetVolumes(struct server *srvp)
      for (vp=afs_volumes[j]; vp; vp=vp->next) {
 	for (k=0; k<MAXHOSTS; k++) {
 	   if (!srvp || (vp->serverHost[k] == srvp)) {
+	      vp->serverHost[k] = 0;
 	      afs_ResetVolumeInfo(vp);
 	      break;
 	   }
@@ -672,7 +673,7 @@ static struct volume *afs_NewVolumeByName(char *aname, afs_int32 acell, int agoo
 	} else
 	    code = -1;
     } while
-      (afs_Analyze(tconn, code, (struct VenusFid *) 0, areq,
+      (afs_Analyze(tconn, code, (struct VenusFid *) 0, &treq,
 		   -1, /* no op code for this */
 		   SHARED_LOCK, tcell));
 
