@@ -7,7 +7,11 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <afsconfig.h>
 #include <afs/param.h>
+
+RCSID("$Header: /cvs/openafs/src/bozo/bosserver.c,v 1.12.2.4 2002/04/21 04:14:50 zacheiss Exp $");
+
 #include <afs/stds.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -24,6 +28,7 @@
 #else
 #include <unistd.h>
 #include <netinet/in.h>
+#include <syslog.h>
 #endif /* AFS_NT40_ENV */
 #include <afs/cellconfig.h>
 #include <rx/rx.h>
@@ -47,6 +52,9 @@ extern int BOZO_ExecuteRequest();
 extern int RXSTATS_ExecuteRequest();
 extern int afsconf_GetKey();
 extern struct bnode_ops fsbnode_ops, ezbnode_ops, cronbnode_ops;
+
+void bozo_Log();
+
 struct afsconf_dir *bozo_confdir = 0;	/* bozo configuration dir */
 static char *bozo_pid;
 struct rx_securityClass *bozo_rxsc[3];
@@ -55,6 +63,10 @@ FILE *bozo_logFile;
 extern int rx_stackSize;    /* for rx_SetStackSize macro */
 
 int DoLogging = 0;
+int DoSyslog = 0;
+#ifndef AFS_NT40_ENV
+int DoSyslogFacility = LOG_DAEMON;
+#endif
 static afs_int32 nextRestart;
 static afs_int32 nextDay;
 
@@ -117,6 +129,20 @@ bozo_ReBozo() {
 	i++;
     }
 
+#ifndef AFS_NT40_ENV
+    /* if syslog logging is on, pass "-syslog" to new bosserver */
+    if (DoSyslog) {
+	char *arg=(char *)malloc(40); /* enough for -syslog=# */
+	if ( DoSyslogFacility != LOG_DAEMON ) {
+	    snprintf(arg, 40, "-syslog=%d", DoSyslogFacility);
+	} else {
+	    strcpy(arg, "-syslog");
+	}
+	argv[i] = arg;
+	i++;
+    }
+#endif
+
     /* null-terminate argument list */
     argv[i] = NULL;
 
@@ -643,9 +669,10 @@ char **envp;
     int noAuth = 0;
     struct ktc_encryptionKey tkey;
     int i;
-    pid_t	newSessionID;
     char namebuf[AFSDIR_PATH_MAX];
-
+#ifndef AFS_NT40_ENV
+    int nofork = 0;
+#endif
 #ifdef	AFS_AIX32_ENV
     struct sigaction nsa;
 
@@ -714,6 +741,19 @@ char **envp;
 	    /* set extra logging flag */
 	    DoLogging = 1;
 	}
+#ifndef AFS_NT40_ENV
+	else if (strcmp(argv[code], "-syslog")==0) {
+	    /* set syslog logging flag */
+	    DoSyslog = 1;
+	} 
+	else if (strncmp(argv[code], "-syslog=",8)==0) {
+	    DoSyslog = 1;
+	    DoSyslogFacility = atoi(argv[code]+8);
+	}
+	else if (strcmp(argv[code], "-nofork")==0) {
+	    nofork = 1;
+	}
+#endif
 	else if (strcmp(argv[code], "-enable_peer_stats")==0) {
 	    rx_enablePeerRPCStats();
 	}
@@ -729,9 +769,17 @@ char **envp;
 
 	    /* hack to support help flag */
 
+#ifndef AFS_NT40_ENV
 	    printf("Usage: bosserver [-noauth] [-log] "
-		   /* "[-enable_peer_stats] [-enable_process_stats] " */
+		   "[-syslog[=FACILITY]] "
+		   "[-enable_peer_stats] [-enable_process_stats] "
+		   "[-nofork] "
 		   "[-help]\n");
+#else
+	    printf("Usage: bosserver [-noauth] [-log] "
+		   "[-enable_peer_stats] [-enable_process_stats] "
+		   "[-help]\n");
+#endif
 	    fflush(stdout);
 
 	    exit(0);
@@ -769,24 +817,28 @@ char **envp;
     /* go into the background and remove our controlling tty */
 
 #ifndef AFS_NT40_ENV
+    if (!nofork)
     background();
 #endif /* ! AFS_NT40_ENV */
 
-    /* switch to logging information to the BosLog file */
-    strcpy(namebuf, AFSDIR_BOZLOG_FILE);
-    strcat(namebuf, ".old");
-    renamefile(AFSDIR_BOZLOG_FILE, namebuf);	/* try rename first */
-    bozo_logFile = fopen(AFSDIR_BOZLOG_FILE, "a");
-    if (!bozo_logFile) {
-	printf("bosserver: can't initialize log file (%s).\n",
-	       AFSDIR_SERVER_BOZLOG_FILEPATH);
-	exit(1);
+    if (!DoSyslog) {
+	strcpy(namebuf, AFSDIR_BOZLOG_FILE);
+	strcat(namebuf, ".old");
+	renamefile(AFSDIR_BOZLOG_FILE, namebuf);    /* try rename first */
+	bozo_logFile = fopen(AFSDIR_BOZLOG_FILE, "a");
+	if (!bozo_logFile) {
+	    printf("bosserver: can't initialize log file (%s).\n",
+		   AFSDIR_SERVER_BOZLOG_FILEPATH);
+	    exit(1);
+	}
+        /* keep log closed normally, so can be removed */
+        fclose(bozo_logFile);
+    } else {
+#ifndef AFS_NT40_ENV
+	openlog("bosserver", LOG_PID, DoSyslogFacility);
+#endif
     }
 
-    /* keep log closed normally, so can be removed */
-
-    fclose(bozo_logFile);
-
     /* Write current state of directory permissions to log file */
     DirAccessOK ();
 
@@ -823,7 +875,7 @@ char **envp;
 	    bozo_Log("try the 'hostname' command\n");
 	    exit(1);
 	}
-	bzero(tcell.hostAddr, sizeof(tcell.hostAddr));	/* not computed */
+	memset(tcell.hostAddr, 0, sizeof(tcell.hostAddr));	/* not computed */
 	code = afsconf_SetCellInfo(bozo_confdir, AFSDIR_SERVER_ETC_DIRPATH, &tcell);
 	if (code) {
 	    bozo_Log("could not create cell database in '%s' (code %d), quitting\n", AFSDIR_SERVER_ETC_DIRPATH, code);
@@ -858,9 +910,8 @@ char **envp;
     bozo_rxsc[2] = (struct rx_securityClass *) rxkad_NewServerSecurityObject(
 					 0, tdir, afsconf_GetKey, (char *) 0);
 
-    /* These two lines disallow jumbograms */
-    rx_maxReceiveSize = OLD_MAX_PACKET_SIZE;
-    rxi_nSendFrags = rxi_nRecvFrags = 1;
+    /* Disable jumbograms */
+    rx_SetNoJumbo();
 
     tservice = rx_NewService(/* port */ 0, /* service id */ 1, 
 		  /*service name */ "bozo", /* security classes */ bozo_rxsc,
@@ -876,35 +927,39 @@ char **envp;
     rx_StartServer(1);	    /* donate this process */
 }
 
+void
 bozo_Log(a,b,c,d,e,f)
 char *a, *b, *c, *d, *e, *f; {
     char tdate[26];
     time_t myTime;
 
-    myTime = time(0);
-    strcpy(tdate, ctime(&myTime));	/* copy out of static area asap */
-    tdate[24] = ':';
+    if (DoSyslog) {
+#ifndef AFS_NT40_ENV
+	syslog(LOG_INFO, a, b, c, d, e, f);
+#endif
+    } else {
+	myTime = time(0);
+	strcpy(tdate, ctime(&myTime));	/* copy out of static area asap */
+	tdate[24] = ':';
 
-    /* log normally closed, so can be removed */
+	/* log normally closed, so can be removed */
 
-    bozo_logFile=fopen(AFSDIR_SERVER_BOZLOG_FILEPATH, "a");
-    if(bozo_logFile == NULL)
-    {
-	printf("bosserver: WARNING: problem with %s", AFSDIR_SERVER_BOZLOG_FILEPATH);
-	fflush(stdout);
+	bozo_logFile=fopen(AFSDIR_SERVER_BOZLOG_FILEPATH, "a");
+	if(bozo_logFile == NULL) {
+	    printf("bosserver: WARNING: problem with %s", AFSDIR_SERVER_BOZLOG_FILEPATH);
+	    fflush(stdout);
+	}
+
+	if (bozo_logFile) {
+	    fprintf(bozo_logFile, "%s ", tdate);
+	    fprintf(bozo_logFile, a, b, c, d, e, f);
+	    fflush(bozo_logFile);
+	} else {
+	    printf("%s ", tdate);
+	    printf(a, b, c, d, e, f);
+	}
+	
+	/* close so rm BosLog works */
+	fclose(bozo_logFile);
     }
-
-    if (bozo_logFile) {
-	fprintf(bozo_logFile, "%s ", tdate);
-	fprintf(bozo_logFile, a, b, c, d, e, f);
-	fflush(bozo_logFile);
-    }
-    else {
-	printf("%s ", tdate);
-	printf(a, b, c, d, e, f);
-    }
-
-    /* close so rm BosLog works */
-
-    fclose(bozo_logFile);
 }
